package com.devplatform.admin.common.utils;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.List;
import java.util.Map;
import org.apache.commons.io.IOUtils;

/**
 * @author Administrator
 */
public class HttpRequestUtil {

  /**
   * 发送http请求取得返回的输入流
   *
   * @param requestUrl 请求地址
   * @return InputStream
   */
  public static InputStream httpRequestIo(String requestUrl) {
    InputStream inputStream = null;
    try {
      URL url = new URL(requestUrl);
      HttpURLConnection httpUrlConn = (HttpURLConnection) url.openConnection();
      httpUrlConn.setDoInput(true);
      httpUrlConn.setRequestMethod("GET");
      httpUrlConn.connect();
      // 获得返回的输入流
      inputStream = httpUrlConn.getInputStream();
    } catch (Exception e) {
      e.printStackTrace();
    }
    return inputStream;
  }

  /**
   * 向指定URL发送GET方法的请求
   *
   * @param url 发送请求的URL
   * @param param 请求参数，请求参数应该是 name1=value1&name2=value2 的形式。
   * @return URL 所代表远程资源的响应结果
   */
  public static String sendGet(String url, String param) {
    String result = "";
    BufferedReader in = null;
    try {
      String urlNameString = url + "?" + param;
      URL realUrl = new URL(urlNameString);
      // 打开和URL之间的连接
      URLConnection connection = realUrl.openConnection();
      // 设置通用的请求属性
      connection.setRequestProperty("accept", "/*");
      connection.setRequestProperty("connection", "Keep-Alive");
      connection.setRequestProperty(
          "user-agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
      // 建立实际的连接
      connection.connect();
      // 获取所有响应头字段
      Map<String, List<String>> map = connection.getHeaderFields();
      // 遍历所有的响应头字段
      for (String key : map.keySet()) {
        System.out.println(key + "—>" + map.get(key));
      }
      // 定义 BufferedReader输入流来读取URL的响应
      in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
      String line;
      while ((line = in.readLine()) != null) {
        result += line;
      }
    } catch (Exception e) {
      System.out.println("发送GET请求出现异常！" + e);
      e.printStackTrace();
    }
    // 使用finally块来关闭输入流
    finally {
      try {
        if (in != null) {
          in.close();
        }
      } catch (Exception e2) {
        e2.printStackTrace();
      }
    }
    return result;
  }

  /**
   * 向指定 URL 发送POST方法的请求
   *
   * @param url 发送请求的 URL
   * @param string 请求参数，请求参数应该是 name1=value1&name2=value2 的形式。
   * @return 所代表远程资源的响应结果
   */
  public static String sendPost(String url, String string) {
    PrintWriter out = null;
    BufferedReader in = null;
    String result = "";
    try {
      URL realUrl = new URL(url);
      // 打开和URL之间的连接
      URLConnection conn = realUrl.openConnection();
      // 设置通用的请求属性
      conn.setRequestProperty("accept", "*/*");
      conn.setRequestProperty("connection", "Keep-Alive");
      conn.setRequestProperty(
          "user-agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
      // 发送POST请求必须设置如下两行
      conn.setDoOutput(true);
      conn.setDoInput(true);
      // 获取URLConnection对象对应的输出流
      out = new PrintWriter(conn.getOutputStream());
      // 发送请求参数
      out.print(string);
      // flush输出流的缓冲
      out.flush();
      // 定义BufferedReader输入流来读取URL的响应
      in = new BufferedReader(new InputStreamReader(conn.getInputStream()));
      String line;
      while ((line = in.readLine()) != null) {
        result += line;
      }
    } catch (Exception e) {
      System.out.println("发送 POST 请求出现异常！" + e);
      e.printStackTrace();
    }
    // 使用finally块来关闭输出流、输入流
    finally {
      try {
        if (out != null) {
          out.close();
        }
        if (in != null) {
          in.close();
        }
      } catch (IOException ex) {
        ex.printStackTrace();
      }
    }
    return result;
  }

  /**
   * 发送Http post请求
   *
   * @param xmlInfo json转化成的字符串
   * @param stringUrl 请求url
   * @return 返回信息
   */
  public static String doHttpPost(String stringUrl, String xmlInfo) {
    byte[] xmlData = xmlInfo.getBytes();
    InputStream instr = null;
    java.io.ByteArrayOutputStream out = null;
    String result = null;
    try {
      URL url = new URL(stringUrl);
      URLConnection urlCon = url.openConnection();
      urlCon.setDoOutput(true);
      urlCon.setDoInput(true);
      urlCon.setUseCaches(false);
      urlCon.setRequestProperty("content-Type", "application/json");
      urlCon.setRequestProperty("charset", "utf-8");
      urlCon.setRequestProperty("Content-length", String.valueOf(xmlData.length));
      DataOutputStream printout = new DataOutputStream(urlCon.getOutputStream());
      printout.write(xmlData);
      printout.flush();
      printout.close();
      instr = urlCon.getInputStream();
      byte[] bis = IOUtils.toByteArray(instr);
      String responseString = new String(bis, "UTF-8");
      if ((responseString == null) || ("".equals(responseString.trim()))) {
        System.out.println("返回空");
      }
      result = responseString;

    } catch (Exception e) {
      e.printStackTrace();
      result = "0";
    } finally {
      try {
        out.close();
        instr.close();

      } catch (Exception ex) {
        result = "0";
      }
    }
    return result;
  }

  public static void main(String[] args) {
    // demo:代理访问
    String url = "https://api.weixin.qq.com/sns/jscode2session";
    //		String para =
    // "code=061cq6Hw0PMRac1ylUFw0Bj8Hw0cq6HQ&appId=wx06c619704536ab15&secret=c241ba1188988ad5be67b2f775a15111&grantType=authorization_code";
    String para =
        "appid=wx06c619704536ab15&secret=c241ba1188988ad5be67b2f775a15111&js_code=061Ap4Bk0D4Fhs15IbAk0txpBk0Ap4Bn&grant_type=authorization_code";

    String sr = HttpRequestUtil.sendGet(url, para);
    System.out.println(sr);
  }
}
